/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.zkoss.poi.ss.formula.ptg;

import org.zkoss.poi.util.LittleEndianInput;
import org.zkoss.poi.util.LittleEndianOutput;
//ZSS-747, ZSS-1317
/**
 * This Ptg is for multiple area expression generally used in chart series and is only used for parsing
 * and evaluation which will not be written out.
 * It doesn't exist in any book.
 * @since 3.9.13
 */
public final class Parenthesis2Ptg extends ControlPtg {

	private final static int SIZE = 3;
	public final static byte sid = 0x1a; // XLS does not use 0x1a sid. We borrow it for MutipleArea representation

	//So we can render the multiple area formula back;
	// e.g. (A1, A2, Sheet1!A3:B3) generally used in chart series
	private final int _numOperands;

	public Parenthesis2Ptg(LittleEndianInput in)  {
		// 2 bytes unused:
		_numOperands = in.readUShort();
	}
	
	public Parenthesis2Ptg(int operands) {
		_numOperands = operands;
	}
	
	public void write(LittleEndianOutput out) {
		out.writeByte(sid + getPtgClass());
		out.writeShort(_numOperands);
	}

	public int getSize() {
		return SIZE;
	}

	public String toFormulaString() {
		return "()";
	}

	public String toFormulaString(String[] operands) {
		StringBuffer sb = new StringBuffer().append("(").append(operands[0]);
		for (int j = 1; j < operands.length; ++j) {
			sb.append(",").append(operands[j]);
		}
		sb.append(")");
		return sb.toString();
	}
	
	public int getNumberOfOperands() {
		return _numOperands;
	}
}
